
math.randomseed(os.time()) for i = 1, 4 do math.random() end
application:setBackgroundColor(0xffffff)

Thing = Core.class(Bitmap)
function Thing:init(texture, x, y, w, h, d, name)
  self.name = name
  self.width = w
  self.height = h
  self.depth = d

  self:moveTo(x, y)

  local speed = math.random(10, 35) -- Each thing moves at a random speed in a random direction
  local angle = math.random(0, 360)
  self.velocity = {x = speed * math.cos(math.rad(angle - 90)), y = speed * math.sin(math.rad(angle - 90))}
  self.isShape = true
end

function Thing:moveTo(x, y) -- Move a thing to the desired location
  self.x2 = x + self.depth
  self.y2 = y + self.depth
  self:setPosition(x, y)
end

Game = Core.class(Sprite)
function Game:init()
  local WIDTH, HEIGHT = application:getContentWidth(), application:getContentHeight()

  self.thingSprites = Sprite.new() -- container sprite to hold our things
  self:addChild(self.thingSprites)

  self.things = {} -- table of all things in the world
  
  -- Create a bunch of things
  table.insert(self.things, Thing.new(Texture.new("images/redthing.png", false), 64, 64, 48, 40, 24, "red") )
  table.insert(self.things, Thing.new(Texture.new("images/greenthing.png", false), 200, 64, 48, 40, 24, "green") )
  table.insert(self.things, Thing.new(Texture.new("images/bluething.png", false), 64, 380, 48, 40, 24, "blue") )
  table.insert(self.things, Thing.new(Texture.new("images/purplething.png", false), 200, 380, 48, 40, 24, "purple") )
  table.insert(self.things, Thing.new(Texture.new("images/yellowthing.png", false), 64, 200, 48, 40, 24, "yellow") )
  table.insert(self.things, Thing.new(Texture.new("images/greything.png", false), 200, 200, 48, 40, 24, "grey") )

  self.bump = require("bump").newWorld() -- Create collision world

  -- Create walls so things can't escape the display
  local walls = { {000, 000, 320, 010, isWall = true}, {000, 470, 320, 010, isWall = true,}, {000, 000, 010, 480, isWall = true,}, {310, 010, 010, 480, isWall = true,} }
  for i = 1, #walls do self.bump:add(walls[i], walls[i][1], walls[i][2], walls[i][3], walls[i][4]) end

 -- Add things to collision world so they can collide
  local things = self.things
  for i = 1, #things do
    local thing = things[i]
    self.bump:add(thing, thing.x2, thing.y2, thing.width, thing.height) -- Each thing can collide with other things (and walls)
  end

  self.currentTimer = os.timer()
  self:addEventListener(Event.ENTER_FRAME, self.update, self)
end

-- This is the main function that does the sorting magic (possibly)
function Game:sortThings()
  local things = self.things
  local sorted = {}
  table.insert(sorted, things[1]) -- Always just insert the first one

 -- Return true if a is behind b
  local function isBehind(a, b)
    if a.x2 + a.y2 < b.x2 + b.y2 then -- Magic
      return true
    end
    return false
  end

 -- Insert thing into table at correct position
  local function insertThing(a)
    for j = 1, #sorted do
      local b = sorted[j]
      if isBehind(a, b) then
        table.insert(sorted, j, a)
		return
      end
    end
    table.insert(sorted, a) -- Must be at the top
  end

 -- Sort the things
  for i = 2, #things do
    local thing = things[i]
	insertThing(thing)
  end

  -- Remove all child sprites from display group
  local group = self.thingSprites
  local nc = group:getNumChildren()
  if nc > 0 then
    for i = nc, 1, -1 do
	  group:removeChildAt(i)
	end
  end

 -- Add sorted sprites to display group
  for i = #sorted, 1, -1 do
    group:addChild(sorted[i])
  end
end

-- Update a things position and bounce it off anything it collides with
 function Game:updateThing(thing, dt)
  local v = thing.velocity
  thing.x2 = thing.x2 + v.x * dt -- Set desired x, y position
  thing.y2 = thing.y2 + v.y * dt
    
  local filter = function(item, other) -- Collision filter
    if other.isWall or other.isShape then return "bounce" end
  end

  local actualX, actualY, cols, len = self.bump:move(thing, thing.x2, thing.y2, filter)
  thing:moveTo(actualX - thing.depth, actualY - thing.depth)
  for i=1,len do
    local col = cols[1]
    local bounciness = 1
    local nx, ny = col.normal.x, col.normal.y
    local vx, vy = v.x, v.y
    if (nx < 0 and vx > 0) or (nx > 0 and vx < 0) then
      vx = -vx * bounciness
    end
    if (ny < 0 and vy > 0) or (ny > 0 and vy < 0) then
      vy = -vy * bounciness
    end
    v.x, v.y = vx, vy
  end
end

-- Main loop
function Game:update(e)
  local timer = os.timer()
  local dt = timer - self.currentTimer
  self.currentTimer = timer

  local things = self.things -- Update all of our things
  for i = 1, #things do
    local thing = things[i]
    self:updateThing(thing, dt)
  end

  self:sortThings() -- Sort them to draw in the right order
end

stage:addChild(Game.new())
